@@ -34,6 +34,7 @@ using FSpot;
 using FSpot.Filters;
 using FSpot.Widgets;
 using FSpot.Utils;
+using FSpot.UI.Dialog;
 
 namespace FSpotFolderExport {
 	public class FolderExport : FSpot.Extensions.IExporter {
@@ -89,7 +90,7 @@ namespace FSpotFolderExport {
 		// FIME this needs to be a real temp directory
 		string gallery_path = Path.Combine (Path.GetTempPath (), "f-spot-original-" + System.DateTime.Now.Ticks.ToString ());
 
-		FSpot.ThreadProgressDialog progress_dialog;
+		ThreadProgressDialog progress_dialog;
 		System.Threading.Thread command_thread;
 
 		public FolderExport ()
@@ -141,8 +142,8 @@ namespace FSpotFolderExport {
 
 			uri_chooser.LocalOnly = false;
 
-			if (Preferences.Get (URI_KEY) != null && Preferences.Get (URI_KEY) as string != String.Empty)
-				uri_chooser.SetUri (Preferences.Get (URI_KEY) as string);
+			if (!String.IsNullOrEmpty (Preferences.Get<string> (URI_KEY)))
+				uri_chooser.SetUri (Preferences.Get<string> (URI_KEY));
 			else
 				uri_chooser.SetFilename (uri_path);
 
@@ -258,7 +259,7 @@ namespace FSpotFolderExport {
 				if (gallery is OriginalGallery) {
 					bool include_tarballs;
 					try {
-						include_tarballs = (bool)Preferences.Get (INCLUDE_TARBALLS_KEY);
+						include_tarballs = Preferences.Get<bool> (INCLUDE_TARBALLS_KEY);
 					} catch (NullReferenceException){
 						include_tarballs = true;
 						Preferences.Set (INCLUDE_TARBALLS_KEY, true);
@@ -292,7 +293,7 @@ namespace FSpotFolderExport {
 				}
 
 				if (open) {
-					GnomeUtil.UrlShow (target.ToString ());
+					GtkBeans.Global.ShowUri (Dialog.Screen, target.ToString ());
 				}
 
 				// Save these settings for next time
@@ -395,53 +396,46 @@ namespace FSpotFolderExport {
 			// 2: zipfiles
 			// 9: directories + info.txt + .htaccess
 			// this should actually be 1 anyway, because we transfer just one dir
-			progress_dialog = new FSpot.ThreadProgressDialog (command_thread, 1);
+			progress_dialog = new ThreadProgressDialog (command_thread, 1);
 			progress_dialog.Start ();
 		}
 
 		void LoadPreference (string key)
 		{
-			object val = Preferences.Get (key);
-
-			if (val == null)
-				return;
-
-			//System.Console.WriteLine ("Setting {0} to {1}", key, val);
-
 			switch (key) {
 			case SCALE_KEY:
-				if (scale_check.Active != (bool) val)
-					scale_check.Active = (bool) val;
+				if (scale_check.Active != Preferences.Get<bool> (key))
+					scale_check.Active = Preferences.Get<bool> (key);
 				break;
 
 			case SIZE_KEY:
-				size_spin.Value = (double) (int) val;
+				size_spin.Value = (double) Preferences.Get<int> (key);
 				break;
 
 			case OPEN_KEY:
-				if (open_check.Active != (bool) val)
-					open_check.Active = (bool) val;
+				if (open_check.Active != Preferences.Get<bool> (key))
+					open_check.Active = Preferences.Get<bool> (key);
 				break;
 
 			case ROTATE_KEY:
-				if (rotate_check.Active != (bool) val)
-					rotate_check.Active = (bool) val;
+				if (rotate_check.Active != Preferences.Get<bool> (key))
+					rotate_check.Active = Preferences.Get<bool> (key);
 				break;
 
 			case EXPORT_TAGS_KEY:
-				if (export_tags_check.Active != (bool) val)
-					export_tags_check.Active = (bool) val;
+				if (export_tags_check.Active != Preferences.Get<bool> (key))
+					export_tags_check.Active = Preferences.Get<bool> (key);
 				break;
 
 			case EXPORT_TAG_ICONS_KEY:
-				if (export_tag_icons_check.Active != (bool) val)
-					export_tag_icons_check.Active = (bool) val;
+				if (export_tag_icons_check.Active != Preferences.Get<bool> (key))
+					export_tag_icons_check.Active = Preferences.Get<bool> (key);
 				break;
 
 			case METHOD_KEY:
-				static_radio.Active = (string) val == "static";
-				original_radio.Active = (string) val == "original";
-				plain_radio.Active = (string) val == "folder";
+				static_radio.Active = (Preferences.Get<string> (key) == "static");
+				original_radio.Active = (Preferences.Get<string> (key) == "original");
+				plain_radio.Active = (Preferences.Get<string> (key) == "folder");
 				break;
 			}
 		}
@@ -558,7 +552,7 @@ namespace FSpotFolderExport {
 
 						bool sharpen;
 						try {
-							sharpen = (bool)Preferences.Get (FolderExport.SHARPEN_KEY);
+							sharpen = Preferences.Get<bool> (FolderExport.SHARPEN_KEY);
 						} catch (NullReferenceException) {
 							sharpen = true;
 							Preferences.Set (FolderExport.SHARPEN_KEY, true);
@@ -1376,8 +1370,9 @@ namespace FSpotFolderExport {
 			if (icon.Height != 52 || icon.Width != 52) {
 				scaled=icon.ScaleSimple(52,52,Gdk.InterpType.Bilinear);
 			} else
-				scaled=icon;
+				scaled=icon.Copy ();
 			scaled.Save (SubdirPath("tags",TagName(tag)), "png");
+			scaled.Dispose ();
 		}
 
 		public string TagPath (Tag tag)
