@@ -49,8 +49,6 @@ namespace FSpot {
 		}
 	}
 	
-	public delegate void IOChannelDataReadyEvent (object sender, DataReadEventArgs args);
-
 	public class IOChannel : System.IO.Stream {
 		private HandleRef handle;
 		
@@ -130,6 +128,9 @@ namespace FSpot {
 			
 			status = g_io_channel_flush (handle, out error);
 
+			if (status != IOStatus.Normal && status != IOStatus.Eof)
+				System.Console.WriteLine ("IOChannel status = {0}", status);
+
 			if (error != IntPtr.Zero)
 				throw new GException (error);
 		}
@@ -176,6 +177,9 @@ namespace FSpot {
 				}
 			}
 
+			if (status != IOStatus.Normal && status != IOStatus.Eof)
+				System.Console.WriteLine ("IOChannel status = {0}", status);
+
 			if (error != IntPtr.Zero)
 				throw new GException (error);
 
@@ -193,9 +197,9 @@ namespace FSpot {
 		// FIXME this should hold more than one source in a table
 		// but I am lazy
 		uint data_ready_source;
-		private IOChannelDataReadyEvent data_ready;
+		private EventHandler<DataReadEventArgs> data_ready;
 		private IOFunc func;
-		public event IOChannelDataReadyEvent DataReady {
+		public event EventHandler<DataReadEventArgs> DataReady {
 			add {
 				data_ready += value;
 				func = new IOFunc (DataReadyHandler);
