@@ -13,20 +13,18 @@
 
 using GLib;
 using Gdk;
-using Gnome;
 using Gtk;
-using GtkSharp;
 using System.Collections;
 using System.IO;
 using System;
 using Mono.Unix;
 
-using FSpot.Widgets;
 using FSpot;
 using FSpot.Utils;
 using FSpot.UI.Dialog;
+using FSpot.Widgets;
 
-public class ImportCommand : FSpot.GladeDialog
+public class ImportCommand : GladeDialog
 {
 	internal class SourceItem : ImageMenuItem
 	{
@@ -188,7 +186,6 @@ public class ImportCommand : FSpot.GladeDialog
 	}
 
 	internal abstract class ImportSource {
-		public object Backend;
 		public Gdk.Pixbuf Icon;
 		public string Name;
 	}
@@ -321,6 +318,7 @@ public class ImportCommand : FSpot.GladeDialog
 	[Glade.Widget] Gtk.OptionMenu source_option_menu;
 	[Glade.Widget] Gtk.ScrolledWindow icon_scrolled;
 	[Glade.Widget] Gtk.ScrolledWindow photo_scrolled;
+	[Glade.Widget] Gtk.CheckButton duplicate_check;
 	[Glade.Widget] Gtk.CheckButton recurse_check;
 	[Glade.Widget] Gtk.CheckButton copy_check;
 	[Glade.Widget] Gtk.Button ok_button;
@@ -343,7 +341,7 @@ public class ImportCommand : FSpot.GladeDialog
 	PhotoStore store;
 	FSpot.Delay step;
 	
-	FSpot.PhotoImageView photo_view;
+	PhotoImageView photo_view;
 	ImportBackend importer;
 	FSpot.Widgets.IconView tray;
 
@@ -431,10 +429,8 @@ public class ImportCommand : FSpot.GladeDialog
 	}
 
 	private bool Step ()
-	{			
-		Photo photo;
-		Pixbuf thumbnail;
-		int count;
+	{	
+	 	StepStatusInfo status_info;		
 		bool ongoing = true;
 
 		if (importer == null)
@@ -443,27 +439,27 @@ public class ImportCommand : FSpot.GladeDialog
 		try {
 			// FIXME this is really just an incredibly ugly way of dealing
 			// with the recursive DoImport loops we sometimes get into
-			ongoing = importer.Step (out photo, out thumbnail, out count);
+			ongoing = importer.Step (out status_info);
 		} catch (ImportException e){
 			System.Console.WriteLine (e);
 			return false;
 		}
 
-		if (photo == null || thumbnail == null) {
+		if (!status_info.IsDuplicate && (status_info.Photo == null || status_info.Thumbnail == null)) {
 			Console.WriteLine ("Could not import file");
 		} else {
 			//icon_scrolled.Visible = true;
-			collection.Add (photo);
-		
+		 	if (!status_info.IsDuplicate)
+				collection.Add (status_info.Photo);
 			//grid.AddThumbnail (thumbnail);
 
 		}
 
-		if (thumbnail != null)
-			thumbnail.Dispose ();
+		if (status_info.Thumbnail != null)
+			status_info.Thumbnail.Dispose ();
 		
-		if (count < total)
-			UpdateProgressBar (count + 1, total);
+		if (status_info.Count < total)
+			UpdateProgressBar (status_info.Count + 1, total);
 
 		if (ongoing && total > 0)
 			return true;
@@ -539,24 +535,26 @@ public class ImportCommand : FSpot.GladeDialog
 	{
 		string path = null;
 
-		CompatFileChooserDialog file_selector =
-			new CompatFileChooserDialog ("Import", this.Dialog,
-						     CompatFileChooserDialog.Action.SelectFolder);
+		FileChooserDialog file_chooser =
+			new FileChooserDialog (Catalog.GetString ("Import"), this.Dialog,
+					FileChooserAction.SelectFolder,
+					Stock.Cancel, ResponseType.Cancel,
+					Stock.Open, ResponseType.Ok);
 
-		file_selector.SelectMultiple = false;
+		file_chooser.SelectMultiple = false;
 
 		if (ImportPath != null)
-			file_selector.Filename = ImportPath;
+			file_chooser.SetFilename (ImportPath);
 		else
-			file_selector.Filename = FSpot.Global.HomeDirectory;
+			file_chooser.SetFilename (FSpot.Global.HomeDirectory);
 
-		int response = file_selector.Run ();
+		int response = file_chooser.Run ();
 
 		if ((ResponseType) response == ResponseType.Ok) {
-			path = file_selector.Filename;
+			path = file_chooser.Filename;
 		}
 
-		file_selector.Destroy ();
+		file_chooser.Destroy ();
 		return path;
 	}
 
@@ -589,17 +587,18 @@ public class ImportCommand : FSpot.GladeDialog
 		this.Dialog.WindowPosition = Gtk.WindowPosition.CenterOnParent;
 		this.Dialog.Response += HandleDialogResponse;
 
- 		if ((int) FSpot.Preferences.Get (FSpot.Preferences.IMPORT_WINDOW_WIDTH) > 0)
- 			this.Dialog.Resize ((int) FSpot.Preferences.Get (FSpot.Preferences.IMPORT_WINDOW_WIDTH), (int) FSpot.Preferences.Get(FSpot.Preferences.IMPORT_WINDOW_HEIGHT));
+ 		if (FSpot.Preferences.Get<int> (FSpot.Preferences.IMPORT_WINDOW_WIDTH) > 0)
+ 			this.Dialog.Resize (FSpot.Preferences.Get<int> (FSpot.Preferences.IMPORT_WINDOW_WIDTH), FSpot.Preferences.Get<int> (FSpot.Preferences.IMPORT_WINDOW_HEIGHT));
 
- 		if ((int) FSpot.Preferences.Get (FSpot.Preferences.IMPORT_WINDOW_PANE_POSITION) > 0)
-			import_hpaned.Position = (int) FSpot.Preferences.Get (FSpot.Preferences.IMPORT_WINDOW_PANE_POSITION);
+ 		if (FSpot.Preferences.Get<int> (FSpot.Preferences.IMPORT_WINDOW_PANE_POSITION) > 0)
+			import_hpaned.Position = FSpot.Preferences.Get<int> (FSpot.Preferences.IMPORT_WINDOW_PANE_POSITION);
 
 	        AllowFinish = false;
 		
 		this.Dialog.DefaultResponse = ResponseType.Ok;
 		
 		//import_folder_entry.Activated += HandleEntryActivate;
+		duplicate_check.Toggled += HandleRecurseToggled;
 		recurse_check.Toggled += HandleRecurseToggled;
 		copy_check.Toggled += HandleRecurseToggled;
 
@@ -615,7 +614,7 @@ public class ImportCommand : FSpot.GladeDialog
 		tray.DisplayTags = false;
 		tray.Show ();
 
-		photo_view = new FSpot.PhotoImageView (collection);
+		photo_view = new PhotoImageView (collection);
 		photo_scrolled.Add (photo_view);
 		photo_scrolled.SetSizeRequest (200, 200);
 		photo_view.Show ();
@@ -624,8 +623,8 @@ public class ImportCommand : FSpot.GladeDialog
 		GtkUtil.ModifyColors (photo_scrolled);
 		GtkUtil.ModifyColors (photo_view);
 
-		photo_view.Pixbuf = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "f-spot", 48, (Gtk.IconLookupFlags)0);
-		photo_view.Fit = true;
+		photo_view.Pixbuf = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "f-spot", 128, (Gtk.IconLookupFlags)0);
+		photo_view.ZoomFit (false);
 			
 		tag_entry = new FSpot.Widgets.TagEntry (MainWindow.Toplevel.Database.Tags, false);
 		tag_entry.UpdateFromTagNames (new string []{});
@@ -720,12 +719,13 @@ public class ImportCommand : FSpot.GladeDialog
 		foreach (string tagname in new_tags) {
 			Tag t = db.Tags.GetTagByName (tagname);
 			if (t == null) {
+				// Note for translators: 'Import Tags' is no command, it means 'Tags used in Import'
 				Category default_category = db.Tags.GetTagByName (Catalog.GetString ("Import Tags")) as Category;
 				if (default_category == null) {
-					default_category = db.Tags.CreateCategory (null, Catalog.GetString ("Import Tags"));
-					default_category.ThemeIconName = "f-spot-imported-xmp-tags.png"; 
+					default_category = db.Tags.CreateCategory (null, Catalog.GetString ("Import Tags"), false);
+					default_category.ThemeIconName = "gtk-new"; 
 				}
-				t = db.Tags.CreateCategory (default_category, tagname) as Tag;
+				t = db.Tags.CreateCategory (default_category, tagname, false) as Tag;
 				db.Tags.Commit (t);
 			}
 
@@ -775,9 +775,13 @@ public class ImportCommand : FSpot.GladeDialog
 		bool recurse = true;
 		if (recurse_check != null)
 			recurse = recurse_check.Active;
+
+		bool detect_duplicates = false;
+		if (duplicate_check != null)
+		 	detect_duplicates = duplicate_check.Active;
 		
 //		importer = new FileImportBackend (store, pathimport, copy, recurse, null);
-		importer = new FileImportBackend (store, pathimport, copy, recurse, null, Dialog);
+		importer = new FileImportBackend (store, pathimport, copy, recurse, detect_duplicates, null, Dialog);
 		AllowFinish = false;
 		
 		total = importer.Prepare ();
@@ -847,3 +851,48 @@ public class ImportCommand : FSpot.GladeDialog
 
 #endif
 }
+
+public class StepStatusInfo {
+	private Photo photo;
+	private Pixbuf thumbnail;
+	private int count;
+	private bool is_duplicate;
+
+	public Photo Photo {
+		get  {
+			return photo; 
+		} 
+	}
+
+	public Pixbuf Thumbnail {
+		get {
+			return thumbnail; 
+		} 
+	}
+
+	public int Count {
+		get {
+			return count; 
+		} 
+	}
+
+	public bool IsDuplicate {
+		get {
+			return is_duplicate; 
+		} 
+	}
+
+	public StepStatusInfo (Photo photo, Pixbuf thumbnail, int count, bool is_duplicate)
+	{
+		this.photo = photo;
+		this.thumbnail = thumbnail;
+		this.count = count;
+		this.is_duplicate = is_duplicate;
+	}
+ 
+	public StepStatusInfo (Photo photo, Pixbuf thumbnail, int count)
+		: this (photo, thumbnail, count, false)
+	{ }
+}
+
+
