@@ -9,7 +9,7 @@
  */
 
 using Gtk;
-using Gnome;
+using GLib;
 using System;
 
 using FSpot.Widgets;
@@ -21,8 +21,8 @@ using Mono.Unix;
 
 namespace FSpot {
 	public class SendEmail : GladeDialog {
+		Window parent_window;
 		PhotoQuery query;
-		Gtk.Window parent_window;
 
 		[Glade.Widget] private ScrolledWindow   tray_scrolled;
 		[Glade.Widget] private Button 		ok_button;
@@ -31,7 +31,6 @@ namespace FSpot {
 							large_size, x_large_size, original_size;
 		[Glade.Widget] private CheckButton 	rotate_check;
 
-		int photo_index;
 		bool clean;
 		long Orig_Photo_Size 	= 0;
 		double scale_percentage = 0.3;
@@ -52,13 +51,16 @@ namespace FSpot {
 		System.Threading.Thread command_thread;
 		IBrowsableCollection selection;
 
-		public SendEmail (IBrowsableCollection selection) : base ("mail_dialog")
+		public SendEmail (IBrowsableCollection selection, Window parent_window) : base ("mail_dialog")
 		{
 			this.selection = selection;
+			this.parent_window = parent_window;
 
-			foreach (Photo p in selection.Items)
-				if (Gnome.Vfs.MimeType.GetMimeTypeForUri (p.DefaultVersionUri.ToString ()) != "image/jpeg")
+			for (int i = 0; i < selection.Count; i++) {
+				Photo p = selection[i] as Photo;
+				if (FileFactory.NewForUri (p.DefaultVersionUri).QueryInfo ("standard::content-type", FileQueryInfoFlags.None, null).ContentType != "image/jpeg")
 					force_original = true;
+			}
 
 			if (force_original) {
 				original_size.Active = true;
@@ -68,7 +70,7 @@ namespace FSpot {
 				large_size.Sensitive = false;
 				x_large_size.Sensitive = false;
 			} else  
-				switch ((int) Preferences.Get (Preferences.EXPORT_EMAIL_SIZE)) {
+				switch (Preferences.Get<int> (Preferences.EXPORT_EMAIL_SIZE)) {
 					case 0 :  original_size.Active = true; break;
 					case 1 :  tiny_size.Active = true; break;
 					case 2 :  small_size.Active = true; break;
@@ -78,7 +80,7 @@ namespace FSpot {
 					default: break;
 				}
 
-			rotate_check.Active = (bool) Preferences.Get (Preferences.EXPORT_EMAIL_ROTATE);
+			rotate_check.Active = Preferences.Get<bool> (Preferences.EXPORT_EMAIL_ROTATE);
 			rotate_check.Sensitive = original_size.Active && tiny_size.Sensitive;
 			
 			tray_scrolled.Add (new TrayView (selection));
@@ -86,9 +88,10 @@ namespace FSpot {
 			Dialog.Modal = false;
 
 			// Calculate total original filesize 
-			foreach (Photo photo in selection.Items) {
+			for (int i = 0; i < selection.Count; i++) {
+				Photo photo = selection[i] as Photo;
 				try {
-					Orig_Photo_Size += (new Gnome.Vfs.FileInfo (photo.DefaultVersionUri.ToString ())).Size;
+					Orig_Photo_Size += FileFactory.NewForUri (photo.DefaultVersionUri).QueryInfo ("standard::size", FileQueryInfoFlags.None, null).Size;
 				} catch {
 				}
 			}
@@ -98,18 +101,18 @@ namespace FSpot {
 
 
 			// Calculate approximate size shrinking, use first photo, and shrink to medium size as base.
-			Photo scalephoto = selection.Items [0] as Photo;			
+			Photo scalephoto = selection [0] as Photo;
 			if (scalephoto != null && !force_original) {
 				
 				// Get first photos file size
-				long orig_size = (new Gnome.Vfs.FileInfo (scalephoto.DefaultVersionUri.ToString ())).Size;
+				long orig_size = FileFactory.NewForUri (scalephoto.DefaultVersionUri).QueryInfo ("standard::size", FileQueryInfoFlags.None, null).Size;
 				
 				FilterSet filters = new FilterSet ();
 				filters.Add (new ResizeFilter ((uint)(sizes [3])));
 				long new_size;
 				using (FilterRequest request = new FilterRequest (scalephoto.DefaultVersionUri)) {
 					filters.Convert (request);
-					new_size = (new Gnome.Vfs.FileInfo (request.Current.ToString ())).Size;
+					new_size = FileFactory.NewForUri (request.Current).QueryInfo ("standard::size", FileQueryInfoFlags.None, null).Size;
 				}
 				
 				if (orig_size > 0) {
@@ -134,7 +137,7 @@ namespace FSpot {
 			}
 
 			NumberOfPictures.Text 	= selection.Count.ToString();
-			TotalOriginalSize.Text 	= SizeUtil.ToHumanReadable (Orig_Photo_Size);
+			TotalOriginalSize.Text 	= GLib.Format.SizeForDisplay (Orig_Photo_Size);
 			
 			UpdateEstimatedSize();
 
@@ -186,7 +189,7 @@ namespace FSpot {
 				else
 					new_approx_total_size = System.Convert.ToInt64(Orig_Photo_Size * avg_scale [new_size_index]);
 
-				approxresult = SizeUtil.ToHumanReadable (new_approx_total_size);
+				approxresult = GLib.Format.SizeForDisplay (new_approx_total_size);
 				ApproxNewSize.Text 	= approxresult;	
 
 		}
@@ -236,14 +239,14 @@ namespace FSpot {
 		
 			progress_dialog = new ProgressDialog (Catalog.GetString ("Preparing email"),
 							      ProgressDialog.CancelButtonType.Stop,
-							      selection.Items.Length, 
+							      selection.Count,
 							      parent_window);
 			
 			size = GetScaleSize(); // Which size should we scale to. 0 --> Original
 			
 			// evaluate mailto command and define attachment args for cli
 			System.Text.StringBuilder attach_arg = new System.Text.StringBuilder ();
-			switch (Preferences.Get (Preferences.GNOME_MAILTO_COMMAND) as string) {
+			switch (Preferences.Get<string> (Preferences.GNOME_MAILTO_COMMAND)) {
 				case "thunderbird %s":
 				case "mozilla-thunderbird %s":
 				case "seamonkey -mail -compose %s":
@@ -280,8 +283,8 @@ namespace FSpot {
 			filters.Add (new UniqueNameFilter (tmp_mail_dir));
 
 
-			foreach (Photo photo in selection.Items) {
-			
+			for (int i = 0; i < selection.Count; i++) {
+				Photo photo = selection [i] as Photo;
 				if ( (photo != null) && (!UserCancelled) ) {
 
 					if (progress_dialog != null)
@@ -303,13 +306,13 @@ namespace FSpot {
 						// Mark the path for deletion
 						tmp_paths.Add (request.Current.LocalPath);
 					} catch (Exception e) {
-						Console.WriteLine("Error preparing {0}: {1}", selection[photo_index].Name, e.Message);
+						Console.WriteLine("Error preparing {0}: {1}", selection[i].Name, e.Message);
 						HigMessageDialog md = new HigMessageDialog (parent_window, 
 											    DialogFlags.DestroyWithParent,
 											    MessageType.Error,
 											    ButtonsType.Close,
 											    Catalog.GetString("Error processing image"), 
-											    String.Format(Catalog.GetString("An error occured while processing \"{0}\": {1}"), selection[photo_index].Name, e.Message));
+											    String.Format(Catalog.GetString("An error occured while processing \"{0}\": {1}"), selection[i].Name, e.Message));
 						md.Run();
 						md.Destroy();
 						UserCancelled = true;
@@ -325,7 +328,7 @@ namespace FSpot {
 			else {		
 				// Send the mail :)
 				string mail_subject = Catalog.GetString("my photos");
-				switch (Preferences.Get (Preferences.GNOME_MAILTO_COMMAND) as string) {
+				switch (Preferences.Get<string> (Preferences.GNOME_MAILTO_COMMAND)) {
 					// openSuSE
 					case "thunderbird %s":
 						System.Diagnostics.Process.Start("thunderbird", " -compose \"subject=" + mail_subject + ",attachment='" + mail_attach + "'\"");
@@ -343,7 +346,7 @@ namespace FSpot {
 						System.Diagnostics.Process.Start("kmail", "  --composer --subject \"" + mail_subject + "\"" + mail_attach);
 					break;
 					default: 
-						GnomeUtil.UrlShow ("mailto:?subject=" + System.Web.HttpUtility.UrlEncode(mail_subject) + mail_attach);
+						GtkBeans.Global.ShowUri (Dialog.Screen, "mailto:?subject=" + System.Web.HttpUtility.UrlEncode(mail_subject) + mail_attach);
 					break;
 				}
 				                
@@ -351,7 +354,7 @@ namespace FSpot {
 				if (tmp_paths.Count > 0) {
 					// Fetch timeout value from preferences. In seconds. Needs to be multiplied with 1000 to get msec
 					uint delete_timeout;
-					delete_timeout = (uint) ( (int) Preferences.Get (Preferences.EXPORT_EMAIL_DELETE_TIMEOUT_SEC) );
+					delete_timeout = (uint) (Preferences.Get<int> (Preferences.EXPORT_EMAIL_DELETE_TIMEOUT_SEC));
 					delete_timeout = delete_timeout * 1000; // to get milliseconds.
 
 					// Start a timer and when it occurs, delete the temp files.
