@@ -16,44 +16,9 @@ using Tao.OpenGl;
 using Cairo;
 using System.Runtime.InteropServices;
 using FSpot.Widgets;
+using FSpot.Utils;
 
 namespace FSpot {
-	// FIXME this class is a hack to have get_data functionality
-	// on cairo 1.0.x
-	public sealed class MemorySurface : ImageSurface {
-		[DllImport ("libfspot")]
-		static extern IntPtr f_image_surface_create (Cairo.Format format, int width, int height);
-		
-		[DllImport ("libfspot")]
-		static extern IntPtr f_image_surface_get_data (IntPtr surface);
-
-		[DllImport ("libfspot")]
-		static extern Cairo.Format f_image_surface_get_format (IntPtr surface);
-
-		public MemorySurface (Cairo.Format format, int width, int height)
-			: this (f_image_surface_create (format, width, height))
-		{
-		}
-
-		public MemorySurface (IntPtr handle) : base (handle, true)
-		{
-			if (Pixels == IntPtr.Zero)
-				throw new ApplicationException ("Missing image data");
-		}
-
-		public IntPtr Pixels {
-			get {
-				return f_image_surface_get_data (Handle);
-			}
-		}
-
-		public Cairo.Format Format {
-			get {
-				return f_image_surface_get_format (Handle);
-			}
-		}
-	}
-
 	public class TextureException : System.Exception {
 		public TextureException (string msg) : base (msg)
 		{
@@ -91,7 +56,7 @@ namespace FSpot {
 		public Texture (Gdk.Pixbuf pixbuf) 
 			: this (pixbuf.Width, pixbuf.Height)
 		{
-			MemorySurface surface = CairoUtils.CreateSurface (pixbuf);
+			MemorySurface surface = MemorySurface.CreateSurface (pixbuf);
 			CopyFromSurface (surface);
 			surface.Destroy ();
 		}
@@ -123,21 +88,21 @@ namespace FSpot {
 			int max_size;
 			Gl.glGetIntegerv (Gl.GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB, out max_size);
 			float scale = (float)Math.Min (1.0, max_size / (double) Math.Max (width, height));
-			System.Console.WriteLine ("max texture size {0} scaling to {1}", max_size, scale);
+			//Log.Debug ("max texture size {0} scaling to {1}", max_size, scale);
 			
-			if (surface.Pixels == IntPtr.Zero)
+			if (surface.DataPtr == IntPtr.Zero)
 				throw new TextureException ("Surface has no data");
 
 			if (surface.Format != Format.Rgb24 && surface.Format != Format.Argb32)
 				throw new TextureException ("Unsupported format type");
 
-			IntPtr pixels = surface.Pixels;
+			IntPtr pixels = surface.DataPtr;
 			IntPtr tmp = IntPtr.Zero;
 			if (scale != 1.0) {
 				int swidth = (int)(width * scale);
 				int sheight = (int) (height * scale);
 				tmp = Marshal.AllocHGlobal (swidth * sheight * 4);
-				System.Console.WriteLine ("scaling image {0} x {1}", swidth, sheight);
+				//LogDebug ("scaling image {0} x {1}", swidth, sheight);
 
 				Glu.gluScaleImage (Gl.GL_BGRA,
 						  width,
@@ -168,18 +133,18 @@ namespace FSpot {
 				Marshal.FreeHGlobal (tmp);
 
 			if (Gl.glGetError () != Gl.GL_NO_ERROR)
-				System.Console.WriteLine ("unable to allocate texture");
+				Log.Warning ("unable to allocate texture");
 			//	throw new TextureException ("Unable to allocate texture resources");
 
 			return texture_id;
 		}
 
-		protected void Close ()
+		void Close ()
 		{
-			System.Console.WriteLine ("Disposing {0} IsTexture {1}", texture_id, Gl.glIsTexture (texture_id));
+			//Log.Debug ("Disposing {0} IsTexture {1}", texture_id, Gl.glIsTexture (texture_id));
 			int [] ids = new int [] { texture_id };
 			Gl.glDeleteTextures (1, ids);
-			System.Console.WriteLine ("Done Disposing {0}", ids [0]);
+			//Log.Debug ("Done Disposing {0}", ids [0]);
 		}
 
 		public void Dispose ()
